@@ -275,7 +275,7 @@ static int yajl_found_boolean(void * ctx, int boolean) {
 }
 
 static int yajl_found_number(void * ctx, const char * numberVal, unsigned int numberLen) {
-    char buf[numberLen+1];
+    char* buf = (char*)malloc(numberLen + 1);
     buf[numberLen] = 0;
     memcpy(buf, numberVal, numberLen);
 
@@ -286,6 +286,7 @@ static int yajl_found_number(void * ctx, const char * numberVal, unsigned int nu
     } else {
         yajl_set_static_value(ctx, rb_cstr2inum(buf, 10));
     }
+    free(buf);
     return 1;
 }
 
@@ -315,15 +316,13 @@ static int yajl_found_hash_key(void * ctx, const unsigned char * stringVal, unsi
 #endif
 
     if (wrapper->symbolizeKeys) {
-        char buf[stringLen+1];
-        memcpy(buf, stringVal, stringLen);
-        buf[stringLen] = 0;
-        VALUE stringEncoded = rb_str_new2(buf);
 #ifdef HAVE_RUBY_ENCODING_H
-        rb_enc_associate(stringEncoded, rb_utf8_encoding());
+        ID id = rb_intern3((const char *)stringVal, stringLen, utf8Encoding);
+        keyStr = ID2SYM(id);
+#else
+        VALUE str = rb_str_new((const char *)stringVal, stringLen);
+        keyStr = rb_str_intern(str);
 #endif
-
-        yajl_set_static_value(ctx, ID2SYM(rb_to_id(stringEncoded)));
     } else {
         keyStr = rb_str_new((const char *)stringVal, stringLen);
 #ifdef HAVE_RUBY_ENCODING_H
@@ -332,8 +331,8 @@ static int yajl_found_hash_key(void * ctx, const unsigned char * stringVal, unsi
           keyStr = rb_str_export_to_enc(keyStr, default_internal_enc);
         }
 #endif
-        yajl_set_static_value(ctx, keyStr);
     }
+    yajl_set_static_value(ctx, keyStr);
     yajl_check_and_fire_callback(ctx);
     return 1;
 }
@@ -879,7 +878,11 @@ static VALUE rb_yajl_json_ext_nil_to_json(int argc, VALUE * argv, VALUE self) {
 static VALUE rb_yajl_encoder_enable_json_gem_ext(VALUE klass) {
     rb_define_method(rb_cHash, "to_json", rb_yajl_json_ext_hash_to_json, -1);
     rb_define_method(rb_cArray, "to_json", rb_yajl_json_ext_array_to_json, -1);
+#ifdef RUBY_INTEGER_UNIFICATION
+    rb_define_method(rb_cInteger, "to_json", rb_yajl_json_ext_fixnum_to_json, -1);
+#else
     rb_define_method(rb_cFixnum, "to_json", rb_yajl_json_ext_fixnum_to_json, -1);
+#endif
     rb_define_method(rb_cFloat, "to_json", rb_yajl_json_ext_float_to_json, -1);
     rb_define_method(rb_cString, "to_json", rb_yajl_json_ext_string_to_json, -1);
     rb_define_method(rb_cTrueClass, "to_json", rb_yajl_json_ext_true_to_json, -1);
